#if !defined  HAVE_PERM_GENUS_H__
#define       HAVE_PERM_GENUS_H__
// This file is part of the FXT library.
// Copyright (C) 2011 Joerg Arndt
// License: GNU General Public License version 3 or later,
// see the file COPYING.txt in the main directory.


#include "ds/bitarray.h"
#include "perm/permq.h"  // count_cycles()

#include "fxttypes.h"

//#include "jjassert.h"


inline ulong perm_genus(const ulong *p, ulong n,  // permutation and length
                        const ulong *pi,  // inverse permutation
                        ulong *cpi,  // for rotated inverse permutation
                        bitarray *B=0)
// The genus g(p) of a permutation p of {1,2,...,n} is defined by
// g(p)=(1/2)*[n+1-z(p)-z(cp')], where p' is the inverse permutation of
// p, c = 234...n1 = (1,2,...,n), and z(q) is the number of cycles of
// the permutation q.
// (taken from http://oeis.org/A177267)
{
    const ulong zp = count_cycles(p, n, B);

    for (ulong j=1; j<n; ++j)  cpi[j-1] = pi[j];
    cpi[n-1] = pi[0];
    const ulong zcpi = count_cycles(cpi, n, B);

//    jjassert( 0==( (n + 1 - zp - zcpi) & 1 )  );
    const ulong gen = (n + 1 - zp - zcpi) / 2;
//    jjassert( gen < n );

    return gen;
}
// -------------------------


#endif  // !defined HAVE_PERM_GENUS_H__
